#include "definiciones.h"

portstream_fd	COMStream; 

/**
 *****************************************************************************
 * Inicializa la aplicacin y el sistema necesarios para el funcionamiento
del programa.
 * @param App Devuelve la direccin de la aplicacin alocatada
 * @param SysVGA Devuelve la direccon del sistema(tarjeta grfica) alocatada (VGA)
 * @see CerrarSystemaMIL()
 ****************************************************************************/
DLL_EXPORT void STD InicializarSystemaMIL (MIL_ID *app,MIL_ID *SysVGA)
{
	MappAlloc (M_DEFAULT,&*app);
	MsysAlloc (M_SYSTEM_VGA,M_DEFAULT,M_NO_DDRAW,&*SysVGA);
}

/**
 *****************************************************************************
 * Desalocata los sistemas y la aplicacin que habiamos alocatado para el 
funcionamiento del programa
 * @param App Direccin de la aplicacin que queremos desalocatar
 * @param SysVGA Direccin del sistema(tarjeta grfica) que queremos desalocatar
 * @param SysMeteor Direccin del sistema(tarjeta grfica) que queremos desalocatar
 para las cmaras)
 * @see InicializarSystemaMIL()
 ****************************************************************************/
DLL_EXPORT void STD CerrarSystemaMIL (MIL_ID app,MIL_ID sysVGA,MIL_ID sysMETEOR)
{
	MsysFree (sysVGA);
	if (sysMETEOR!=0) MsysFree (sysMETEOR);
	MappFree (app);
}

/**
 *****************************************************************************
 * Alocata el sistema(tarjeta grfica), el digitalizador y el buffer necesarios 
 para la grabacin de imagenes con las cmaras
 * @param WindowHandle Handle de la ventana donde se visualizara las imagenes
capatadas por las camaras
 * @param SysMeteor Devuelve la direccin del sistema(tarjeta grfica) alocatada (METEOR
para las cmaras)
 * @param Disp Devuelve la direccin del display alocatado para ver lo que captan las cmaras
 * @param Camara Devuelve la direccin del buffer donde se almacena lo que captan las cmaras
 * @param Digitalizer Devuelve la direccin del digitalizador de imagenes
 * @param alto Altura de la ventana donde se representara las imagenes captadas
por las cmaras
 * @param ancho Ancho de la ventana donde se representara las imagenes captadas
por las cmaras
 * @see DesconectarCamaraMIL()
 * @see CaptarImagenCamaraMIL()
*****************************************************************************/
DLL_EXPORT void STD ConectarCamaraMIL (HWND WindowHandle,MIL_ID *SysMETEOR,MIL_ID *disp,
									   MIL_ID *camara,MIL_ID *digitalizer,long alto, 
									   long ancho)
{

	MsysAlloc (M_SYSTEM_METEOR,M_DEFAULT,M_NO_DDRAW,&*SysMETEOR);
	MdispAlloc (*SysMETEOR,M_DEFAULT,"M_DEFAULT",M_DEFAULT,&*disp);
	MdigAlloc (*SysMETEOR,M_DEFAULT,"M_CCIR",M_DEFAULT,&*digitalizer);
	MdigChannel(*digitalizer, M_CH0);
//	MdigControl(*digitalizer, M_GRAB_SCALE_X, M_FILL_DISPLAY);
//	MdigControl(*digitalizer, M_GRAB_SCALE_Y, M_FILL_DISPLAY);
	MdigControl(*digitalizer, M_GRAB_SCALE, 0.5);
	MdigInquire(*digitalizer, M_SIZE_X, &ancho);
	MdigInquire(*digitalizer, M_SIZE_Y, &alto);
	MbufAllocColor (*SysMETEOR,3,alto,ancho,8 + M_UNSIGNED,M_IMAGE+M_GRAB+M_DISP,&*camara);
	MdispSelectWindow (*disp,*camara,WindowHandle);
//	MdigGrabContinuous (*digitalizer,*camara);
}

/**
 ***********************************************************************************
 * Desalocata todo lo referente a la grabacin de imagenes con las camaras. Esto es el display, el digitalizador y el buffer
 * @param Display Direccin del display que queremos desalocatar
 * @param Digitalizer Direccin del digitalizador que queremos desalocatar
 * @param Buffer Direccin del buffer que queremos desalocatar
 * @see ConectarCamaraMIL()
 * @see CaptarImagenCamaraMIL()
 **********************************************************************************/
DLL_EXPORT void STD	DesconectarCamaraMIL (MIL_ID Display, MIL_ID Digitalizer, MIL_ID Buffer)
{
	MdispFree (Display);
	MdigFree (Digitalizer);
	MbufFree (Buffer);
}

/**
 ****************************************************************************
 * Capta una imagen de la cmara conectada al digitalizador especificado
 * @param Digitalizer Digitalizador que se encarga de captar la imagen
 * @param Buffer Buffer donde se almacena la imagen captada por el digitalizador
 * @see ConectarCamaraMIL()
 * @see DesconectarCamaraMIL()
 *****************************************************************************/
DLL_EXPORT void STD CaptarImagenCamaraMIL (MIL_ID Digitalizer, MIL_ID Buffer)
{
	MdigGrab (Digitalizer,Buffer);
}

/**
 *****************************************************************************
 * Establece conexin con el servidor que esta funcionando en el PC del robot 
para el control del Pan&Tilt.
 * @param IP Direccin IP del ordenador donde esta conectado el Pan&Tilt
 * @see DesconectarPAN_TILT
 * @return Devuelve un 1 si ha conectado y un 0 si no lo ha hecho
 ***************************************************************************/
DLL_EXPORT long STD ConectarPAN_TILT (char *IP)
{
	SetIP (IP);
	COMStream=open_host_port("COM2");
	if (COMStream==(void *)0xcccccccc) {return 0;}
	else { 
		set_mode(SPEED_CONTROL_MODE,PTU_PURE_VELOCITY_SPEED_CONTROL_MODE);
		return 1; 
	};
}
/**
 ******************************************************************************
 * Desconecta el socket al que estamos conectados para el control del Pan&Tilt
 * @see ConectarPAN_TILT
 *****************************************************************************/
DLL_EXPORT void STD DesconectarPAN_TILT (void)
{
	close_host_port(COMStream);
}

/**
 *******************************************************************************
 * Hace girar el Pan&Tilt hacia la derecha a velocidad constante
 * @param Velocidad Velocidad de giro
 * @see PanIzquierda()
 * @see TiltArriba()
 * @see TiltAbajo()
 * @see DetenerPan()
 * @see DetenerTilt()
 ******************************************************************************/
DLL_EXPORT void STD PanDerecha(long Velocidad)
{
	PTU_PARM_PTR dato;

	if (Velocidad==0) {halt(PAN);}
	dato=-(PTU_PARM_PTR)Velocidad;
	set_desired(PAN,SPEED,&dato,ABSOLUTE);
}

/**
 *******************************************************************************
 * Hace girar el Pan&Tilt hacia la izquierda a velocidad constante
 * @param Velocidad Velocidad de giro
 * @see PanDerecha()
 * @see TiltArriba()
 * @see TiltAbajo()
 * @see DetenerPan()
 * @see DetenerTilt()
 ******************************************************************************/
DLL_EXPORT void STD PanIzquierda(long Velocidad)
{
	PTU_PARM_PTR dato;

	if (Velocidad==0) {halt(PAN);}
	dato=(PTU_PARM_PTR)Velocidad;
	set_desired(PAN,SPEED,&dato,ABSOLUTE);
}

/**
 *******************************************************************************
 * Hace girar el Pan&Tilt hacia arriba a velocidad constante
 * @param Velocidad Velocidad de giro
 * @see PanDerecha()
 * @see PanIzquierda()
 * @see TiltAbajo()
 * @see DetenerPan()
 * @see DetenerTilt()
 ******************************************************************************/
DLL_EXPORT void STD TiltArriba(long Velocidad)
{
	PTU_PARM_PTR dato;

	if (Velocidad==0) {halt(TILT);}
	dato=(PTU_PARM_PTR)Velocidad;
	set_desired(TILT,SPEED,&dato,ABSOLUTE);
}

/**
 *******************************************************************************
 * Hace girar el Pan&Tilt hacia abajo a velocidad constante
 * @param Velocidad Velocidad de giro
 * @see PanDerecha()
 * @see PanIzquierda()
 * @see TiltArriba()
 * @see DetenerPan()
 * @see DetenerTilt()
 ******************************************************************************/
DLL_EXPORT void STD TiltAbajo(long Velocidad)
{
	PTU_PARM_PTR dato;

	if (Velocidad==0) {halt(TILT);}
	dato=-(PTU_PARM_PTR)Velocidad;
	set_desired(TILT,SPEED,&dato,ABSOLUTE);
}

/**
 *******************************************************************************
 * Detiene el giro del Pan
 * @see PanDerecha()
 * @see PanIzquierda()
 * @see TiltArriba()
 * @see TiltAbajo()
 * @see DetenerTilt()
 ******************************************************************************/
DLL_EXPORT void STD DetenerPan() { halt(PAN);}

/**
 *******************************************************************************
 * Detiene el giro del Tilt
 * @see PanDerecha()
 * @see PanIzquierda()
 * @see TiltArriba()
 * @see TiltAbajo()
 * @see DetenerPan()
 ******************************************************************************/
DLL_EXPORT void STD DetenerTilt() { halt(TILT);}

/**
 *******************************************************************************
 * Devuelve la Posicin del Pan en pasos (un paso = 0.0514285 grados)
 * @see PosicionTILT
 * @return Posicin del Pan en pasos
 ******************************************************************************/
DLL_EXPORT long STD PosicionPAN ()
{
	FlushInputBuffer(COMStream);
	return get_current(PAN,POSITION);
}

/**
 *******************************************************************************
 * Devuelve la Posicin del Tilt en pasos (un paso = 0.0514285 grados)
 * @see PosicionPAN
 * @return Posicin del Tilt en pasos
 ******************************************************************************/
DLL_EXPORT long STD PosicionTILT ()
{
	FlushInputBuffer(COMStream);
	return get_current(TILT,POSITION);	
}

/**
 *******************************************************************************
 * Devuelve la velocidad actual del Pan
 * @return Velocidad del Pan
 ******************************************************************************/
DLL_EXPORT long STD VelocidadPAN ()
{
	long	velocidad;
	FlushInputBuffer(COMStream);
	velocidad=get_current(PAN,SPEED);
	return velocidad;
}

/**
 *******************************************************************************
 * Gira el ngulo del Pan hasta un ngulo determinado y luego se para
 * @param N Angulo en grados hasta el que girar el Pan
 * @param Velocidad Velocidad a la que queremos que se ejecute el giro del Pan
 * @see GirarTiltHastaAnguloN
 ******************************************************************************/
DLL_EXPORT void STD GirarPanHastaAnguloN (long N, long velocidad)
{
	short dato,vel;
	dato=(short)N;
	vel=(short)velocidad;
	set_mode(SPEED_CONTROL_MODE,PTU_INDEPENDENT_SPEED_CONTROL_MODE);
	set_desired(PAN,SPEED,&vel,ABSOLUTE);
	set_desired(PAN,POSITION,&dato,ABSOLUTE);
	set_mode(SPEED_CONTROL_MODE,PTU_PURE_VELOCITY_SPEED_CONTROL_MODE);
}

/**
 *******************************************************************************
 * Gira el ngulo del Tilt hasta un ngulo determinado y luego se para
 * @param N Angulo en grados hasta el que girar el Pan
 * @param Velocidad Velocidad a la que queremos que se ejecute el Pan
 * @see GirarPanHastaAnguloN
 ******************************************************************************/
DLL_EXPORT void STD GirarTiltHastaAnguloN (long N, long velocidad)
{
	short dato,vel;
	dato=(short)N;
	vel=(short)velocidad;
	set_mode(SPEED_CONTROL_MODE,PTU_INDEPENDENT_SPEED_CONTROL_MODE);
	set_desired(PAN,SPEED,&vel,ABSOLUTE);
	set_desired(TILT,POSITION,&dato,ABSOLUTE);
	set_mode(SPEED_CONTROL_MODE,PTU_PURE_VELOCITY_SPEED_CONTROL_MODE);
}


//////////////////////////////////////////////////////////////////////////
DLL_EXPORT void STD LanzarThreadPanTilt (long Operacion, long Velocidad)
{
	ValoresPT.Operacion=Operacion;
	ValoresPT.Velocidad=Velocidad;
	_beginthread (ThreadMoverPanTilt,0,(void*)&ValoresPT);
}

void ThreadMoverPanTilt (void *Datos)
{
	PassToThreadPanTilt* Val = (PassToThreadPanTilt*)Datos;
	switch ((*Val).Operacion)
	{
	case 0:
		PanDerecha ((*Val).Velocidad);
		break;
	case 1:
//		PanIzquierda ((*Val).Velocidad);
		PTU_PARM_PTR dato;

		if ((*Val).Velocidad==0) {halt(PAN);}
		dato=(PTU_PARM_PTR)(*Val).Velocidad;
		set_desired(PAN,SPEED,&dato,ABSOLUTE);
		break;
	case 2:
		TiltArriba ((*Val).Velocidad);
		break;
	case 3:
		TiltAbajo ((*Val).Velocidad);
		break;
	case 4:
		DetenerPan();
		break;
	case 5:
		DetenerTilt();
		break;
	}
   _endthread();
}
///////////////////////////////////////////////////////////////////////////


