#include "saphira.h"
#include "windows.h"
#include "process.h"
#include "io.h"
#include "fcntl.h"

#define paiRobotBase sfRobot /* ver53 used to refer to Pioneer as flakey */

#define DLL_EXPORT extern "C" __declspec(dllexport)
#define DLL_IMPORT extern "C" __declspec(dllimport)
#define STD __stdcall
#define	ESCALA	0.089473

////////////////////////////////////////////////////////////
//    Definicion de las estructuras 
////////////////////////////////////////////////////////////


/**
 ****************************************************************
 * Estructura de datos utilizada para el paso de datos a un Thread.
 Esta estructura transmite los datos necesarios al Thread encargado
 de seguir una Trayectoria
 ****************************************************************/
typedef struct {
	/** Coordenada X del punto de la trayectoria al que debe dirigirse el robot a continuacin */
	double x;
	/** Coordenada Y del punto de la trayectoria al que debe dirigirse el robot a continuacin */
	double y;
	/** Orientacin; debe ser siempre cero */
	double th;
	/** Establece la velocidad a la que el robot debe diriguirse hacia el punto */
	double Velocidad;
	/** Radio que define el rea alrededor del punto */
	double RadioExito;
}PassToThreadType;

/**
 ****************************************************************
 * Estructura que contiene los parmetros de configuracin del
 comportamiento "Velocidad Constante"
 ****************************************************************/
typedef struct {
	/** Prioridad del comportamiento */
	double Prioridad;
	/** Tiempo que permanecer el comportamiento activo */
	double TimeOut;
	/** Estado del comportamiento en el momento de cargarlo en memoria, 
	Activo o Pausado*/
	double Suspendido;
	/** Velocidad que debe mantener el robot */
	double Velocidad;
}ConstantVelocityType;

/**
 ****************************************************************
 * Estructura que contiene los parmetros de configuracin del
 comportamiento "Mantener Distancias"
 ****************************************************************/
typedef struct {
	/** Prioridad del comportamiento */
	double Prioridad;
	/** Tiempo que permanecer el comportamiento activo */
	double TimeOut;
	/** Estado del comportamiento en el momento de cargarlo en memoria, 
	Activo o Pausado*/
	double Suspendido;
	/** Velocidad de desplazamiento de robot en presencia de objetos */
	double VelocidadCauta;
	/** Sensibilidad ante la presencia de objetos */
	double Sensibilidad;
}KeepOffType;

/**
 ****************************************************************
  * Estructura que contiene los parmetros de configuracin del
 comportamiento "Evitar Colisin"
****************************************************************/
typedef struct {
	/** Prioridad del comportamiento */
	double Prioridad;
	/** Tiempo que permanecer el comportamiento activo */
	double TimeOut;
	/** Estado del comportamiento en el momento de cargarlo en memoria, 
	Activo o Pausado*/
	double Suspendido;
	/** Sensibilidad ante la presencia de objetos frontales */
	double SensibilidadFrontal;
	/** Sensibilidad ante la presencia de objetos laterales */
	double SensibilidadLateral;
	/** Controla la velocidad de giro del robot ante la presencia de obstculos */
	double GananciaGiro;
	/** Determina el radio de seguridad del robot */
	double RadioSeguridad;
}AvoidCollisionType;

/**
 ****************************************************************
  * Estructura que contiene los parmetros de configuracin del
 comportamiento "Stop"
****************************************************************/
typedef struct {
	/** Prioridad del comportamiento */
	double Prioridad;
	/** Tiempo que permanecer el comportamiento activo */
	double TimeOut;
	/** Estado del comportamiento en el momento de cargarlo en memoria, 
	Activo o Pausado*/
	double Suspendido;
}StopType;

/**
 ****************************************************************
 * Esta estructura contiene la informacin de todos los comportamientos
 ****************************************************************/
typedef struct {
	AvoidCollisionType AvoidCollisionParams;
	KeepOffType KeepOffParams;
	ConstantVelocityType ConstantVelocityParams;
	StopType StopParams;
}BehaviorsParams;

////////////////////////////////////////////////////////////
//		Variables Globales
///////////////////////////////////////////////////////////

sfprocess			*ComportamientoVel=NULL,*ComportamientoColision=NULL;
bool				ThreadFinished=false;
PassToThreadType	Valores;

//////////////////////////////////////////////////////////////
//		Cabeceras de las funciones
/////////////////////////////////////////////////////////////

DLL_EXPORT long STD ConexionConRobot (long TipoConexion, double XPosInic ,
									  double YPosInic, double Heading, char *DireccionIP);
DLL_EXPORT void STD DesconexionRobot (void);
DLL_EXPORT void STD GirarRobotDerecha (long velocidad);
DLL_EXPORT void STD GirarRobotIzquierda (long velocidad);
DLL_EXPORT void STD GirarRobotHastaAnguloN (double Angulo);
DLL_EXPORT void STD DetenerGiroRobot (void);
DLL_EXPORT void STD AvanzarRobot (long velocidad);
DLL_EXPORT void STD RetrocederRobot (long velocidad);
DLL_EXPORT void STD DetenerTraslacionRobot (void);
DLL_EXPORT void STD GetRobotPosition (double *PosX,double *PosY,double *Heading);
DLL_EXPORT void STD CargarBehaviors (BehaviorsParams *Parametros);
DLL_EXPORT void STD DescargarBehaviors ();
DLL_EXPORT void STD Behav_Loaded (char *NombreFichero, char *NombreInstancia);
DLL_EXPORT void STD Behav_Kill (char *NombreInstancia);
DLL_EXPORT void STD Behav_Resume (char *NombreInstancia);
DLL_EXPORT void STD Behav_Pause (char *NombreInstancia);
DLL_EXPORT void STD SeguirSpline (long x,long y,long th,double vel,double radio);
DLL_EXPORT int STD ThreadState (void);

int paiRobotStartup(int channel, char *name, HANDLE hInst, int nCmdShow);
void myStartupFn(void);
void myConnectFn(void);
void ThreadSeguirSpline (void *Datos);

