#ifndef _SFUTILS_H
#define _SFUTILS_H

#include "saphira.h"

// --------------------------
// Definici del tipus vector
// --------------------------

typedef struct 
{
    float *telem;               /* Taula d'elements */
    int dim;                    /* Dimensio */
} Vector;

// --------------------------
// Definici del tipus llista
// --------------------------

typedef struct datos      
{
  point *punt;
  struct datos *siguiente;
} telemento;

typedef struct
{
	telemento **punts;
	int *numpunts;
}LlistaPunts;

// -------------------------------
// Definici de funcions de Vector
// -------------------------------

extern "C" __declspec(dllimport) Vector CrearVector(int n);
extern "C" __declspec(dllimport) void DestruirVector(Vector v);
extern "C" __declspec(dllimport) void InicialitzarVector(Vector v);

extern "C" __declspec(dllimport) int DimVector(Vector v);
extern "C" __declspec(dllimport) float ConsVector(Vector v, int i);
extern "C" __declspec(dllimport) void AssigVector(Vector v, int i, float f);
extern "C" __declspec(dllimport) void EscriureVector(Vector v);

extern "C" __declspec(dllimport) LlistaPunts ObtenirLlistaVectors(Vector x, Vector y, Vector angle);  
extern "C" __declspec(dllimport) LlistaPunts ObtenirLlistaVectorsGlobals(Vector x, Vector y, Vector angle);  

// -------------------------------
// Definici de funcions de Llista
// -------------------------------

extern "C" __declspec(dllimport) LlistaPunts llCrearLlistaPunts(void);
extern "C" __declspec(dllimport) void llDestruirLlistaPunts(LlistaPunts l);

extern "C" __declspec(dllimport) int llNumPunts(LlistaPunts l);

extern "C" __declspec(dllimport) void llAfegirPunt(LlistaPunts l, float x, float y, float angle);
extern "C" __declspec(dllimport) void llAfegirPuntGlobal(LlistaPunts l, float x, float y, float angle);

extern "C" __declspec(dllimport) void llInsertarPunt(LlistaPunts l, float x, float y, float angle, int pos);
extern "C" __declspec(dllimport) void llInsertarPuntGlobal(LlistaPunts l, float x, float y, float angle, int pos);

extern "C" __declspec(dllimport) void llEliminarPunt(LlistaPunts l, int pos);
extern "C" __declspec(dllimport) void llEliminarPrimerPunt(LlistaPunts l);
extern "C" __declspec(dllimport) void llEliminarUltimPunt(LlistaPunts l);
extern "C" __declspec(dllimport) void llEliminarTotsPunts(LlistaPunts l);

extern "C" __declspec(dllimport) void llCambiarPunt(LlistaPunts l, int pos, float x, float y, float angle);
extern "C" __declspec(dllimport) void llCambiarPuntGlobal(LlistaPunts l, int pos, float x, float y, float angle);

extern "C" __declspec(dllimport) float llObtenirXPunt(LlistaPunts l, int pos);
extern "C" __declspec(dllimport) float llObtenirYPunt(LlistaPunts l, int pos);
extern "C" __declspec(dllimport) float llObtenirAnglePunt(LlistaPunts l, int pos);

extern "C" __declspec(dllimport) float llObtenirXGlobalPunt(LlistaPunts l, int pos);
extern "C" __declspec(dllimport) float llObtenirYGlobalPunt(LlistaPunts l, int pos);
extern "C" __declspec(dllimport) float llObtenirAngleGlobalPunt(LlistaPunts l, int pos);

extern "C" __declspec(dllimport) Vector llObtenirVectorXLlista(LlistaPunts l);
extern "C" __declspec(dllimport) Vector llObtenirVectorYLlista(LlistaPunts l);
extern "C" __declspec(dllimport) Vector llObtenirVectorAngleLlista(LlistaPunts l);

extern "C" __declspec(dllimport) Vector llObtenirVectorXGlobalLlista(LlistaPunts l);
extern "C" __declspec(dllimport) Vector llObtenirVectorYGlobalLlista(LlistaPunts l);
extern "C" __declspec(dllimport) Vector llObtenirVectorAngleGlobalLlista(LlistaPunts l);

extern "C" __declspec(dllimport) void llEscriureLlistaPunts(LlistaPunts l);
extern "C" __declspec(dllimport) void llEscriureLlistaPuntsGlobals(LlistaPunts l);

// ---------------------
// Funcions geomtriques
// ---------------------

extern "C" __declspec(dllimport) LlistaPunts CalcularSpline(LlistaPunts l, Vector knots, int k, int num_punts);
extern "C" __declspec(dllimport) LlistaPunts CalcularStandardSpline(LlistaPunts l, int k, int num_punts);

extern "C" __declspec(dllimport) LlistaPunts CalcularBezier(LlistaPunts l, int k, int num_punts);

extern "C" __declspec(dllimport) int MinimaDistanciaLlistaPunts(LlistaPunts l);
extern "C" __declspec(dllimport) int MaximaDistanciaLlistaPunts(LlistaPunts l);

extern "C" __declspec(dllimport) float llDistanciaPunt(LlistaPunts l, int pos);
extern "C" __declspec(dllimport) float llAnglePunt(LlistaPunts l, int pos);

extern "C" __declspec(dllimport) float llLongitudLlista(LlistaPunts l);
extern "C" __declspec(dllimport) float llLongitudLlistaInterval(LlistaPunts l, int inici, int fi); 
// Si es vol inici 0 es pot posar -1, si es vol final ltim posar -1)

#endif